'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = Emoji;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _emojibase = require('emojibase');

var _emoji = require('../data/emoji');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright   2016, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 * 
 */

var LARGE_MULTIPLIER = 3;

function Emoji(_ref) {
  var emojiLargeSize = _ref.emojiLargeSize,
      emojiPath = _ref.emojiPath,
      emojiSize = _ref.emojiSize,
      emoticon = _ref.emoticon,
      enlargeEmoji = _ref.enlargeEmoji,
      shortcode = _ref.shortcode,
      unicode = _ref.unicode;

  if ("production" !== process.env.NODE_ENV) {
    if (!emoticon && !shortcode && !unicode) {
      throw new Error('Emoji component requires a `unicode` character, `emoticon`, or a `shortcode`.');
    }
  }

  if (!unicode && shortcode) {
    unicode = _emoji.SHORTCODE_TO_UNICODE[shortcode];
  }

  if (!unicode && emoticon) {
    unicode = _emoji.EMOTICON_TO_UNICODE[emoticon];
  }

  if (!unicode || !_emoji.EMOJIS[unicode]) {
    return _react2.default.createElement(
      'span',
      null,
      unicode || emoticon || shortcode
    );
  }

  var emoji = _emoji.EMOJIS[unicode];
  var className = ['interweave__emoji'];
  var styles = {};

  if (emojiSize) {
    styles.display = 'inline-block';
    styles.verticalAlign = 'middle';
    styles.width = emojiSize + 'em';
  }

  if (enlargeEmoji) {
    className.push('interweave__emoji--large');

    if (emojiLargeSize) {
      styles.width = emojiLargeSize + 'em';
    } else if (emojiSize) {
      styles.width = emojiSize * LARGE_MULTIPLIER + 'em';
    }
  }

  var path = emojiPath || '{{hexcode}}';

  if (typeof path === 'function') {
    path = path(emoji.hexcode, enlargeEmoji, emojiSize, emojiLargeSize);
  } else {
    path = path.replace('{{hexcode}}', emoji.hexcode);
  }

  return _react2.default.createElement('img', {
    src: path,
    alt: unicode,
    title: emoji.annotation || '',
    style: styles,
    className: className.join(' '),
    'aria-label': emoji.annotation || '',
    'data-emoticon': emoji.emoticon || '',
    'data-unicode': unicode,
    'data-hexcode': emoji.hexcode,
    'data-shortcodes': _emoji.UNICODE_TO_SHORTCODES[unicode].join(', '),
    'data-codepoint': (0, _emojibase.fromHexcodeToCodepoint)(emoji.hexcode).join('-')
  });
}

Emoji.propTypes = {
  emojiLargeSize: _propTypes2.default.number,
  emojiPath: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),
  emojiSize: _propTypes2.default.number,
  emoticon: _propTypes2.default.string,
  enlargeEmoji: _propTypes2.default.bool,
  shortcode: _propTypes2.default.string,
  unicode: _propTypes2.default.string
};

Emoji.defaultProps = {
  emojiLargeSize: 0,
  emojiPath: '{{hexcode}}',
  emojiSize: 0,
  emoticon: '',
  enlargeEmoji: false,
  shortcode: '',
  unicode: ''
};